package zendesk.messaging.android

import android.content.Context
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.SupervisorJob
import kotlinx.coroutines.cancelChildren
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.launch
import kotlinx.coroutines.sync.Mutex
import kotlinx.coroutines.sync.withLock
import zendesk.logger.Logger
import zendesk.messaging.android.internal.MessagingCredentials
import zendesk.messaging.android.internal.MessagingFactory
import zendesk.messaging.android.internal.MessagingScreen
import zendesk.messaging.android.internal.StubMessaging
import zendesk.messaging.android.internal.conversationscreen.ConversationActivity

/**
 * Public entry point of the SDK. To use [Messaging] out of the box you must call [Messaging.initialize] first and once
 * [SuccessCallback] is called the SDK instance will be available through [Messaging.instance].
 *
 * ```kotlin
 * Messaging.initialize(
 *     context = context,
 *     channelKey = "{your-channel-key}",
 *     successCallback = {
 *         // The initialization was successful
 *     },
 *     failureCallback = { cause ->
 *         // The initialization failed and "cause" is an optional "MessagingError"
 *     }
 * )
 * ```
 *
 * After [Messaging.initialize] completes successfully, calling [Messaging.instance] will return an instance of
 * [Messaging] which you can use to perform SDK actions such as showing the conversation screen.
 *
 * ```kotlin
 * Messaging.instance().showMessaging(context)
 * ```
 *
 * @see Messaging.initialize
 * @see Messaging.instance
 */
public interface Messaging {

    public companion object {
        internal const val LOG_TAG = "Messaging"

        private val supervisorJob = SupervisorJob()
        private val scope: CoroutineScope = CoroutineScope(Dispatchers.Main + supervisorJob)

        /**
         * A [Mutex] to be used in conjunction with [initialize] to avoid multiple instances from being created by
         * mistake.
         */
        private val initializeMutex = Mutex()

        /**
         * Singleton instance of [Messaging], for the convenience of integrators that only wish to use a single
         * instance of [Messaging] out of the box.
         */
        private var messaging: Messaging? = null

        /**
         * Backing mutable version of [messagingDelegate].
         */
        private val mutableMessagingDelegate = MutableStateFlow(MessagingDelegate())

        /**
         * Internal [Flow] of [MessagingDelegate] that [Messaging] can observe
         */
        internal val messagingDelegate: Flow<MessagingDelegate> = mutableMessagingDelegate

        /**
         * Backing mutable version of [currentlyVisibleScreen].
         */
        private val mutableCurrentlyVisibleScreen = MutableStateFlow<MessagingScreen?>(null)

        /**
         * Internal [StateFlow] of nullable [MessagingScreen] that can be observed
         */
        internal val currentlyVisibleScreen: StateFlow<MessagingScreen?>
            get() = mutableCurrentlyVisibleScreen

        /**
         * Initializes [Messaging] for the given [channelKey], calling [successCallback] if the initialization
         * completes successfully and [failureCallback] if it fails.
         *
         * After a successful call to [initialize], calling it again will simply call [successCallback] immediately.
         *
         * This function is synchronized and won't run concurrently if called more than once at the same time.
         *
         * @param context an instance of [Context]. It will be retained for as long as this instance of [Messaging]
         * exists so avoid providing a short lived [Context] which could result in memory leaks
         * @param channelKey the key of your Messaging for Android channel
         * @param successCallback an implementation of [SuccessCallback] to be notified of a successful result
         * @param failureCallback an implementation of [FailureCallback] to be notified of a failure
         */
        @JvmStatic
        public fun initialize(
            context: Context,
            channelKey: String,
            successCallback: SuccessCallback<Messaging>,
            failureCallback: FailureCallback<MessagingError>,
        ) {
            scope.launch {
                initializeMutex.withLock {
                    messaging?.let {
                        Logger.i(LOG_TAG, "Messaging.initialize was already called, returning early.")
                        successCallback.onSuccess(it)
                    } ?: run {
                        val credentials = MessagingCredentials.builder(channelKey = channelKey).build()

                        when (val result = MessagingFactory.from(context).create(credentials)) {
                            is MessagingResult.Success -> {
                                messaging = result.value
                                successCallback.onSuccess(result.value)
                            }
                            is MessagingResult.Failure -> failureCallback.onFailure(result.cause)
                        }
                    }
                }
            }
        }

        /**
         * Retrieves the current instance of [Messaging].
         *
         * If [Messaging.initialize] was not called or haven't completed yet, this method will return a stubbed version
         * of [Messaging] that will only output logs when any of its methods are called.
         *
         * @return the instance of [Messaging] if [Messaging.initialize] completed successfully, a stub otherwise
         *
         * @see Messaging.initialize
         */
        @JvmStatic
        public fun instance(): Messaging = messaging ?: StubMessaging

        /**
         * Invalidates the current instance of [Messaging].
         *
         * After calling this method you will have to call [Messaging.initialize] again if you would like to use
         * [Messaging].
         */
        @JvmStatic
        public fun invalidate() {
            supervisorJob.cancelChildren()
            messaging = null
        }

        /**
         * Sets the [MessagingDelegate] that should be used by [Messaging].
         *
         * @param messagingDelegate an implementation of [MessagingDelegate], or null if you want to remove
         * one that was previously set
         */
        @JvmStatic
        public fun setDelegate(messagingDelegate: MessagingDelegate?) {
            mutableMessagingDelegate.value = messagingDelegate ?: MessagingDelegate()
        }

        // region Currently visible Messaging screen
        /**
         * Extension function that allows an instance of [MessagingScreen] to mark itself
         * as the currently visible Messaging screen.
         *
         * @receiver an instance of [MessagingScreen]
         */
        internal fun MessagingScreen.setAsVisibleMessagingScreen() {
            mutableCurrentlyVisibleScreen.value = this
        }

        /**
         * Extension function that allows an instance of [MessagingScreen] to clear itself
         * as the currently visible Messaging screen.
         *
         * @receiver an instance of [MessagingScreen]
         */
        internal fun MessagingScreen.clearAsVisibleMessagingScreen() {
            if (mutableCurrentlyVisibleScreen.value == this) {
                mutableCurrentlyVisibleScreen.value = null
            }
        }
        // endregion
    }

    /**
     * Given the [context], starts the [ConversationActivity] immediately.
     *
     * @param context the [Context] to start the activity
     */
    public fun showMessaging(context: Context)

    /**
     * Given the [context] and [intentFlags], starts the [ConversationActivity] immediately.
     *
     * @param context the [Context] to start the activity
     * @param intentFlags special flags controlling how the intent to start the [ConversationActivity] is
     * handled, 0 by default
     */
    public fun showMessaging(context: Context, intentFlags: Int)
}
