package zendesk.messaging.android

/**
 * [Messaging]-related errors that can happen and be returned in a [MessagingResult.Failure].
 *
 * @property message a message describing this error
 */
public sealed class MessagingError(override val message: String) : Throwable(message) {

    /**
     * [Messaging.initialize] or [MessagingFactory.create] was called but an invalid channel key was provided.
     */
    public object InvalidChannelKey : MessagingError(
        message = "The provided channelKey is invalid."
    )

    /**
     * If a [MessagingResult] was expected out of this call, none was received.
     */
    public object NoResultReceived : MessagingError(
        message = "No MessagingResult to return for this call."
    )

    /**
     * [Messaging.instance] was called before [Messaging.initialize] completed successfully.
     */
    public object NotInitialized : MessagingError(
        message = "Messaging.instance() was called before initialization was completed."
    )

    /**
     * [Messaging.initialize] or [MessagingFactory.create] was called but the SDK is not enabled for the provided
     * subdomain URL and channel identifier.
     */
    public object SdkNotEnabled : MessagingError(
        message = "The SDK is not enabled for this integration."
    )

    /**
     * [Messaging.initialize] or [MessagingFactory.create] was called but no account was found for the provided
     * subdomain URL and channel identifier.
     */
    public object AccountNotFound : MessagingError(
        message = "No account found for the provided credentials."
    )

    /**
     * [Messaging.initialize] or [MessagingFactory.create] was called but no configuration was received from the server.
     */
    public object MissingConfiguration : MessagingError(
        message = "The configuration for this Messaging integration could not be retrieved."
    )

    /**
     * [Messaging.initialize] or [MessagingFactory.create] was called but it failed to initialize with all of its
     * dependencies.
     */
    public object FailedToInitialize : MessagingError(
        message = "The provided parameters were not valid to create an instance of Messaging."
    )

    /**
     * [Messaging] was unable to find the conversation to display.
     */
    public object ConversationNotFound : MessagingError(
        message = "There was a problem retrieving the conversation."
    )
}
