package zendesk.messaging.android.push

import com.google.firebase.messaging.FirebaseMessagingService
import com.google.firebase.messaging.RemoteMessage
import zendesk.logger.Logger

/**
 * A default implementation of [FirebaseMessagingService] for use with Messaging SDK. This service isn't
 * automatically added to the Android Manifest to provide flexibility in case there are any integrations
 * that require firebase in your app other than Messaging.
 *
 * If Messaging is the only expected source of push notifications then this service can be registered in
 * the android manifest to handle the messages on your behalf. Add the following inside the application tag:
 * ```
 * <service android:name="zendesk.messaging.android.push.DefaultMessagingService"
 *     android:exported="false">
 *     <intent-filter>
 *         <action android:name="com.google.firebase.MESSAGING_EVENT" />
 *     </intent-filter>
 * </service>
 * ```
 */
public class DefaultMessagingService : FirebaseMessagingService() {

    private companion object {
        private const val LOG_TAG = "DefaultMessagingService"
    }

    override fun onMessageReceived(remoteMessage: RemoteMessage) {
        when (PushNotifications.shouldBeDisplayed(remoteMessage.data)) {
            PushResponsibility.MESSAGING_SHOULD_DISPLAY -> {
                Logger.d(LOG_TAG, "Notification received from Messaging and should be displayed")
                PushNotifications.displayNotification(context = this, messageData = remoteMessage.data)
            }
            PushResponsibility.MESSAGING_SHOULD_NOT_DISPLAY -> {
                Logger.d(LOG_TAG, "Notification received from Messaging but shouldn't be displayed")
            }
            PushResponsibility.NOT_FROM_MESSAGING -> {
                Logger.d(LOG_TAG, "Notification received but doesn't belong to Messaging")
            }
        }
    }

    override fun onNewToken(newToken: String) {
        Logger.d(LOG_TAG, "New Firebase Cloud Messaging token: $newToken")
        PushNotifications.updatePushNotificationToken(newToken)
    }
}
